require "#{File.dirname(__FILE__)}/../lib/spec_helper.rb"

describe 'The Dashboard' do

  before :all do
    visit_dashboard

    # wait for nodes to load
    session.should have_css("#content", :visible => true)
  end

  after :all do
    logout
  end

  describe 'node management' do

    s_class_name_01 = 'sample::cLASS01'
    s_class_name_02 = 'sample::class02'
    s_group_name_01 = 'sample::gROUP01'
    s_group_name_02 = 'sample::group02'

    before :all do
      create_class(s_class_name_01) 
      create_class(s_class_name_02) 
      create_group(s_group_name_01)
      create_group(s_group_name_02)
    end

    after :all do
      remove_class(s_class_name_01) 
      remove_class(s_class_name_02) 
      remove_group(s_group_name_01)
      remove_group(s_group_name_02)
    end

    it 'should add and remove a simple node' do
      node_name = 'test::node'
      create_node(node_name)
      should_have_node(node_name)
      remove_node(node_name)
      should_not_have_node(node_name)
    end

    describe 'advanced insert' do

      node_name         = 'test::node'
      node_description  = 'Description of test::node'
      params            = [{ :key => 'k1', :value => 'v1'}, { :key => 'k2', :value => 'v2'}]
 
      before :all do
        create_advanced_node(node_name, node_description, params, s_class_name_01.downcase, s_group_name_01.downcase)
      end

      after :all do
        remove_node(node_name)
      end

      it 'should add a node' do
        should_have_node(node_name)
      end

      it 'should have the set description' do
        select_node(node_name)
        session.should have_css('div#main div[class*="description"] p', :text => node_description)
      end

      it 'should have the added variables' do
        select_node(node_name)
        show_should_have_variables(params)
      end

      it 'should have the set class' do
        select_node(node_name)
        classes_table = get_inspector_table('Classes')
        classes_table.should have_css('a', :text => s_class_name_01)
      end

      it 'should have the set group' do
        select_node(node_name)
        classes_table = get_inspector_table('Groups')
        classes_table.should have_css('a', :text => s_group_name_01)
      end
    end

    describe 'advanced modification' do

      node_name         = 'test::node'
      node_description  = 'Description of test::node'
      params            = [{ :key => 'k1', :value => 'v1'}, { :key => 'k2', :value => 'v2'}]
 
      node_description_new  = 'Description of test::nodenew'
      params_new            = [{ :key => 'k1', :value => 'v0'}]
 
      before :all do
        create_advanced_node(node_name, node_description, params, s_class_name_01.downcase, s_group_name_01.downcase)
        modify_advanced_node(node_description_new, params_new, s_class_name_02.downcase, s_group_name_02.downcase)
      end

      after :all do
        remove_node(node_name)
      end

      it 'should have a node' do
        should_have_node(node_name)
      end

      it 'should have the set description' do
        select_node(node_name)
        session.should have_css('div#main div[class*="description"] p', :text => node_description_new)
      end

      it 'should have the added variables' do
        select_node(node_name)
        show_should_have_variables(params_new)
      end

      it 'should have the new class' do
        select_node(node_name)
        classes_table = get_inspector_table('Classes')
        classes_table.should have_css('a', :text => s_class_name_02)
      end

      it 'should not have the old class' do
        select_node(node_name)
        classes_table = get_inspector_table('Classes')
        classes_table.should have_no_css('a', :text => s_class_name_01)
      end

      it 'should have the new group' do
        select_node(node_name)
        classes_table = get_inspector_table('Groups')
        classes_table.should have_css('a', :text => s_group_name_02)
      end

      it 'should not have the old group' do
        select_node(node_name)
        classes_table = get_inspector_table('Groups')
        classes_table.should have_no_css('a', :text => s_group_name_01)
      end
    end
  end

end

def create_advanced_node(node_name, node_description, params, ref_class_name, ref_group_name)
  session.find('#sidebar').click_link('Add node')
  form_node = session.find('#param_form', :visible => true)

  form_node.fill_in('node_name', :with => "#{node_name}")
  form_node.fill_in('node_description', :with => "#{node_description}")

  # add variables
  add_variables(params)

  # set class
  set_field_with_suggestion('class', ref_class_name)

  # set group
  set_field_with_suggestion('group', ref_group_name)

  form_node.click_button('Create')
  wait_for_modification
end

def modify_advanced_node(node_description, params, ref_class_name, ref_group_name)
  edit_button_css = '#main a.edit.button'
  session.find(edit_button_css).click()

  form_node = session.find('#param_form', :visible => true)
  form_node.fill_in('node_description', :with => "#{node_description}")

  # modify variables
  remove_variables
  add_variables(params)

  # replace class
  remove_field_with_suggestion_values('class')
  set_field_with_suggestion('class', ref_class_name)

  # replace group
  remove_field_with_suggestion_values('group')
  set_field_with_suggestion('group', ref_group_name)

  form_node.click_button('Update')
  wait_for_modification
end
